import dataclasses
import threading
from typing import Optional, List

import smartcard.Exceptions
from py_acr122u.nfc import Reader


class MyReader(Reader):
    def __init__(self):
        super().__init__()

    def apply_settings(self):
        self.set_auto_polling(False)
        self.mute_buzzer()
        self.reset_lights()

    def green_light_on(self):
        self.led_control(0b00001010, 0, 0, 0, 0)

    def red_light_on(self):
        self.led_control(0b00000101, 0, 0, 0, 0)

    def scanned_beep(self):
        self.led_control(0b10000000, 1, 0, 1, 0x01)

    def error_beep(self):
        self.led_control(0b01010000, 2, 2, 3, 0x03)

    def confirm_beep(self):
        self.led_control(0b10100000, 1, 1, 2, 0x01)

    def read_uuid(self) -> Optional[List[int]]:
        try:
            data = self.pn532.in_auto_poll(0x05, 0x01, 0x10)
            if len(data) < 5:
                return None
            length = data[9]
            uuid = data[10: 10 + length]
            return uuid
        except smartcard.Exceptions.CardConnectionException:
            return None


@dataclasses.dataclass
class ReaderContainer():
    lock: threading.Lock = dataclasses.field(default_factory=threading.Lock)
    reader: Optional[MyReader] = None



import logging
import secrets
from typing import List

from flask import Flask, render_template
from flask_socketio import SocketIO, emit

from .my_reader import ReaderContainer
from .reader_helpers import CardReaderConnector, CardReaderPoller

app = Flask(__name__)

if app.debug:
    logging.basicConfig(level=logging.DEBUG)
else:
    app.config['SECRET_KEY'] = secrets.token_urlsafe(16)
    logging.basicConfig(level=logging.INFO)

socketio = SocketIO(app, async_mode='threading')

card_reader_container = ReaderContainer()

card_reader_connector = CardReaderConnector(card_reader_container)
card_reader_connector.daemon = True


def emit_uuid(uuid: List[int]):
    socketio.emit('card_scanned', {'uuid': uuid})


card_reader_poller = CardReaderPoller(card_reader_container, emit_uuid)
card_reader_poller.daemon = True


@app.route('/test')
def test_page():
    return render_template('test.html')


@socketio.on('connect')
def connect(auth):
    app.logger.info("Client connected")


@socketio.on('disconnect')
def disconnect():
    app.logger.info("Client disconnected")


@socketio.on('polling')
def polling(start_stop):
    if card_reader_container.reader is None:
        emit('polling', 'no card reader connected')
        return

    if start_stop == 'start':
        card_reader_poller.start_polling()
        emit('polling', 'polling started')
    elif start_stop == 'stop':
        card_reader_poller.stop_polling()
        emit('polling', 'polling stopped')
    else:
        emit('polling', 'not a valid message')


@socketio.on('status_indicator')
def set_status_indicator(status):
    if card_reader_container.reader is None:
        emit('status indicator', 'no card reader connected')
        return

    if status == "confirm":
        with card_reader_container.lock:
            card_reader_container.reader.confirm_beep()
        emit('status indicator', 'confirm status set')
    elif status == "error":
        with card_reader_container.lock:
            card_reader_container.reader.error_beep()
        emit('status indicator', 'error status set')
    else:
        emit('polling', 'not a valid status')


card_reader_connector.start()
card_reader_poller.start()

if __name__ == '__main__':
    socketio.run(app, host='0.0.0.0', port=8080)



import threading
from time import sleep
from typing import Callable, List

from acr122u_websocket.my_reader import MyReader, ReaderContainer


class NoCardReaderConnected(Exception):
    pass


class CardReaderConnector(threading.Thread):
    def __init__(self, card_reader_container: ReaderContainer):
        super().__init__()
        self._card_reader_container = card_reader_container

    def run(self):
        while True:
            while self._card_reader_container.reader is None:
                try:
                    with self._card_reader_container.lock:
                        self._card_reader_container.reader = MyReader()
                        self._card_reader_container.reader.connect()
                        self._card_reader_container.reader.apply_settings()
                except Exception as e:
                    print(e)
                    self._card_reader_container.reader = None
                sleep(0.1)
            if self._card_reader_container.reader is not None:
                try:
                    self._card_reader_container.reader.get_uid()
                except Exception as e:
                    print(e)
                    self._card_reader_container.reader = None
            sleep(1)


class CardReaderPoller(threading.Thread):
    def __init__(self, card_reader_container: ReaderContainer, uuid_callback: Callable[[List[int]], None]):
        super().__init__()
        self._card_reader_container = card_reader_container
        self._uuid_callback = uuid_callback
        self._polling: bool = False
        self._current_uuid = None
        self._last_uuid = None

    def run(self):
        while True:
            while self._card_reader_container.reader is not None:
                if self._polling:
                    try:
                        with self._card_reader_container.lock:
                            self._current_uuid = self._card_reader_container.reader.read_uuid()

                            if self._current_uuid is not None and self._current_uuid != self._last_uuid:
                                self._card_reader_container.reader.scanned_beep()
                                self._uuid_callback(self._current_uuid)

                            self._last_uuid = self._current_uuid
                    except Exception as e:
                        print(e)
                sleep(0.05)
            sleep(1)

    def start_polling(self):
        if self._polling:
            return

        self._current_uuid = None
        self._last_uuid = None

        with self._card_reader_container.lock:
            if self._card_reader_container.reader is None:
                raise NoCardReaderConnected()
            self._card_reader_container.reader.green_light_on()

        self._polling = True

    def stop_polling(self):
        if not self._polling:
            return

        with self._card_reader_container.lock:
            if self._card_reader_container.reader is None:
                raise NoCardReaderConnected()
            self._card_reader_container.reader.reset_lights()

        self._polling = False

        self._current_uuid = None
        self._last_uuid = None
